@@ -3,7 +3,7 @@
  *
  * Copyright (C) 2005-2012  NTT DATA CORPORATION
  *
- * Version: 0.2.6   2020/04/13
+ * Version: 0.2.7   2020/08/20
  */
 
 #include "internal.h"
@@ -623,6 +623,9 @@ char *cs_get_exe(void)
 	mm = current->mm;
 	if (!mm)
 		goto task_has_no_mm;
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(4, 1, 0)
+	exe_file = get_mm_exe_file(mm);
+#else
 	down_read(&mm->mmap_sem);
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 26)
 	/* Not using get_mm_exe_file() as it is not exported. */
@@ -639,6 +642,7 @@ char *cs_get_exe(void)
 	if (exe_file)
 		get_file(exe_file);
 	up_read(&mm->mmap_sem);
+#endif
 	if (exe_file) {
 		char *cp = cs_realpath(&exe_file->f_path);
 
