@@ -3,7 +3,7 @@
  *
  * Copyright (C) 2005-2012  NTT DATA CORPORATION
  *
- * Version: 0.2.6   2020/04/13
+ * Version: 0.2.7   2020/08/20
  */
 
 #ifndef _LINUX_CAITSITH_H
@@ -29,7 +29,13 @@ struct sock;
 struct sk_buff;
 struct msghdr;
 struct pid_namespace;
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(3, 8, 0)
+struct cs_request_info;
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(5, 8, 0)
+/*
+ * Don't depend on prototype definition, for exec_binprm() is a static function
+ * which is implicitly inlined by compiler.
+ */
+#elif LINUX_VERSION_CODE >= KERNEL_VERSION(3, 8, 0)
 int search_binary_handler(struct linux_binprm *bprm);
 #else
 int search_binary_handler(struct linux_binprm *bprm, struct pt_regs *regs);
@@ -150,7 +156,10 @@ struct caitsith_operations {
 				   struct dentry *dentry);
 	int (*sigqueue_permission)(pid_t pid, int sig);
 	int (*tgsigqueue_permission)(pid_t tgid, pid_t pid, int sig);
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(3, 8, 0)
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(5, 8, 0)
+	int (*start_execve) (struct linux_binprm *bprm, struct cs_request_info **rp);
+	void (*finish_execve) (int retval, struct cs_request_info *r);
+#elif LINUX_VERSION_CODE >= KERNEL_VERSION(3, 8, 0)
 	int (*search_binary_handler)(struct linux_binprm *bprm);
 #else
 	int (*search_binary_handler)(struct linux_binprm *bprm,
@@ -389,7 +398,23 @@ static inline int ccs_chmod_permission(struct dentry *dentry,
 	return func ? func(dentry, mnt, mode) : 0;
 }
 
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(3, 8, 0)
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(5, 8, 0)
+
+/* Define as a macro here, for exec_binprm() is a static function. */
+#define ccs_exec_binprm(bprm)\
+	({\
+	struct cs_request_info *r = NULL;	\
+	void (*func) (int, struct cs_request_info *r);\
+	int retval = caitsith_ops.start_execve(bprm, &r);\
+	if (!retval)\
+		retval = exec_binprm(bprm);\
+	func = caitsith_ops.finish_execve;\
+	if (func)\
+		func(retval, r);\
+	retval;\
+	})
+
+#elif LINUX_VERSION_CODE >= KERNEL_VERSION(3, 8, 0)
 
 static inline int ccs_search_binary_handler(struct linux_binprm *bprm)
 {
@@ -574,7 +599,12 @@ static inline int ccs_chmod_permission(struct dentry *dentry,
 	return 0;
 }
 
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(3, 8, 0)
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(5, 8, 0)
+
+/* Define as a macro here, for exec_binprm() is a static function. */
+#define ccs_exec_binprm(bprm) exec_binprm(bprm)
+
+#elif LINUX_VERSION_CODE >= KERNEL_VERSION(3, 8, 0)
 
 static inline int ccs_search_binary_handler(struct linux_binprm *bprm)
 {
