@@ -3,7 +3,7 @@
  *
  * Copyright (C) 2005-2012  NTT DATA CORPORATION
  *
- * Version: 0.2.6   2020/04/13
+ * Version: 0.2.7   2020/08/20
  */
 
 #include "caitsith.h"
@@ -171,7 +171,41 @@ static void cs_unescape(unsigned char *dest);
 
 /***** SECTION4: Standalone functions section *****/
 
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 36)
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(5, 8, 0)
+
+/**
+ * prepare_binprm - Read the first BINPRM_BUF_SIZE bytes.
+ *
+ * @bprm: Pointer to "struct linux_binprm".
+ *
+ * Same with prepare_binprm() in fs/exec.c
+ */
+static inline int prepare_binprm(struct linux_binprm *bprm)
+{
+	loff_t pos = 0;
+
+	memset(bprm->buf, 0, BINPRM_BUF_SIZE);
+	return kernel_read(bprm->file, bprm->buf, BINPRM_BUF_SIZE, &pos);
+}
+
+/**
+ * cs_copy_argv - Wrapper for copy_string_kernel().
+ *
+ * @arg:  String to copy.
+ * @bprm: Pointer to "struct linux_binprm".
+ *
+ * Returns return value of copy_string_kernel().
+ */
+ static inline int cs_copy_argv(const char *arg, struct linux_binprm *bprm)
+ {
+	 const int ret = copy_string_kernel(arg, bprm);
+
+	 if (ret >= 0)
+		 bprm->argc++;
+	 return ret;
+ }
+
+#elif LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 36)
 
 /**
  * cs_copy_argv - Wrapper for copy_strings_kernel().
@@ -783,7 +817,11 @@ bool cs_dump_page(struct linux_binprm *bprm, unsigned long pos,
 	}
 	/* Same with get_arg_page(bprm, pos, 0) in fs/exec.c */
 #ifdef CONFIG_MMU
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(4, 10, 0)
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(5, 9, 0)
+	if (get_user_pages_remote(bprm->mm, pos, 1, FOLL_FORCE, &page,
+				  NULL, NULL) <= 0)
+		return false;
+#elif LINUX_VERSION_CODE >= KERNEL_VERSION(4, 10, 0)
 	if (get_user_pages_remote(current, bprm->mm, pos, 1, FOLL_FORCE, &page,
 				  NULL, NULL) <= 0)
 		return false;
@@ -1992,7 +2030,11 @@ int cs_socket_post_accept_permission(struct socket *sock,
 static bool cs_kernel_service(void)
 {
 	/* Nothing to do if I am a kernel service. */
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 27)
+	return current->flags & PF_KTHREAD;
+#else
 	return segment_eq(get_fs(), KERNEL_DS);
+#endif
 }
 
 #endif
